-- This program is open source, licensed under the PostgreSQL License.
-- For license terms, see the LICENSE file.
--
-- Copyright (C) 2023: Julien Rouhaud

-- complain if script is sourced in psql, rather than via CREATE EXTENSION
\echo Use "CREATE EXTENSION pg_decode_record" to load this file. \quit

CREATE FUNCTION pg_decode_record(IN record bytea,
    IN spcoid oid, IN relfilenode oid,
    OUT decoded_record text
)
    LANGUAGE C STRICT VOLATILE COST 100
AS '$libdir/pg_decode_record', 'pg_decode_record';

CREATE FUNCTION pg_decode_all_records(IN path text,
    IN ignore_toast bool DEFAULT TRUE)
RETURNS SETOF text
AS $$
DECLARE
    f text;
    mydbid oid;
    spcoid oid;
    datid oid;
    relfilenode oid;
    record bytea;
    decoded text;
BEGIN
    SELECT oid INTO mydbid
    FROM pg_catalog.pg_database
    WHERE datname = pg_catalog.current_database();

    FOR f IN SELECT * FROM pg_catalog.pg_ls_dir(path) ORDER BY 1
    LOOP
        IF f !~ '[0-9A-F]{8}-[0-9A-F]{8}\.[0-9]+\.[0-9]+.[0-9]+.\d+_[a-z]+' THEN
            RAISE EXCEPTION 'File "%" has unexpected name, directory "%" is '
                            'probably wrong', f, path;
        END IF;
        record = pg_catalog.pg_read_binary_file(path || '/' || f);
        spcoid = split_part(f, '.', 2)::oid;
        datid = split_part(f, '.', 3)::oid;
        relfilenode = split_part(f, '.', 4)::oid;

        CONTINUE WHEN datid != mydbid;

        decoded := pg_decode_record(record, spcoid, relfilenode);

        -- Ignore TOAST records if callers said so
        CONTINUE WHEN ignore_toast AND decoded ILIKE 'INSERT INTO pg_toast%';

        RETURN NEXT decoded;
    END LOOP;
END;
$$
LANGUAGE plpgsql STRICT;
